package ngc.typetest;

import java.awt.*;
import java.awt.event.*;
import java.util.*;

/** this class handles the characters as the user types them
 *  Note that Pasting has to be disabled!!
 */

public class TestArea extends TextArea {
	protected TestDocument testDoc=null;
	protected String title="";
	protected int textLoc=0;
	protected String nextWord="";
	protected StringBuffer currBuf=null;
	protected boolean timerStart=false, testComplete=false, isOwnFile=false;
	protected int wordCount=0, wrongCount=0, correctCount=0;
	protected long startTime=0;
	protected Vector wrongWord=new Vector();
	protected Dialog resultDialog=null;
	public void reset() {
		if (testComplete==false)
			endTest();
		textLoc=0;
		currBuf.setLength(0);
		timerStart=false;
		testComplete=false;
		wordCount=0; wrongCount=0; correctCount=0;
		startTime=0;
		wrongWord.removeAllElements();
		setText("Start typing to begin");
		StatPanel.reset();
		nextWord=testDoc.getFirstWord();
	}
	public TestArea(TestDocument tdoc) {
		super("Start typing to begin", 16, 80, TextArea.SCROLLBARS_VERTICAL_ONLY);
		testDoc=tdoc;
		nextWord=testDoc.getFirstWord();
		currBuf=new StringBuffer();

		setEditable(true);

		enableEvents(AWTEvent.KEY_EVENT_MASK);
		enableEvents(AWTEvent.MOUSE_EVENT_MASK);
	}
	public void changeFont() {
		Font currFont=getFont();
		System.out.println(currFont.getSize()+"\t"+currFont.getName());
		Font newFont=currFont.deriveFont(Font.BOLD, 15.0f);
		setFont(newFont);
	}

	public void processMouseEvent(MouseEvent evt) {
		requestFocusInWindow();
		evt.consume();
	}
	public void setCaretPosition(int loc) {
		System.out.println("setCaretPosition: "+loc);
	}
	public void processKeyEvent(KeyEvent e) {
		if (testComplete) {
			e.consume();
			return;
		}
		if ((e.getID() == KeyEvent.KEY_TYPED)) {
			if (timerStart==false) {
				timerStart=true;
				setText("");
				startTime=System.currentTimeMillis();
				StatPanel.initTimer();
				wrongWord.removeAllElements();
			}
			char keyTyped=e.getKeyChar();
			if (keyTyped=='\b') {
				doBackspace(e);
			}
			else if (keyTyped == ' ') {
				if (currBuf.length()>0)
					getNextWord();
			}
			else if (keyTyped=='\n') {
				if (currBuf.length()>0)
					getNextWord();
			}
			else currBuf.append(keyTyped);
		}
		else {
			int keyCode=e.getKeyCode();
			if (keyCode==8)
				displayBackspace(e);
			else
			if (isEndKey(keyCode))
				showStatistics();
			else if (ignoreKey(keyCode) || e.isControlDown() || e.isAltDown() || e.isMetaDown())
				e.consume();
			else {
				super.processKeyEvent(e);
			}
		}
	}
	public void displayBackspace(KeyEvent e) {
		int len=currBuf.length();
		if (len>0)
			super.processKeyEvent(e);
		else	e.consume();
	}
	public static void testChar(char ch) {
		int intVal=(int)ch;
		System.out.println(intVal+"\t"+"["+ch+"] "+Character.getType(ch));
	}
	public void doBackspace(KeyEvent evt) {
		int len=currBuf.length();
		if (len>0) {
			currBuf.setLength(len-1);
		}
		else evt.consume();
	}
	public void getNextWord() {
		if (nextWord==null)
			return;
		String wordTyped=currBuf.toString();
		if (nextWord.equals(wordTyped))
			correctCount++;	
		else {
			wrongCount++;
			System.out.println(nextWord+"]\t["+wordTyped+"]");
			wrongWord.addElement(nextWord+"\t"+wordTyped);
		}
		nextWord=testDoc.getNextWord();
		currBuf.setLength(0);
		wordCount++;
		if (nextWord==null)
			endTest();

	}
	public void endTest() {
		if (timerStart==false)	// had not started test, so no need to end
			return;
		showStatistics();
		setStatistics();
		StatPanel.stopTimer();
		testComplete=true;
		showWrongWords();
		showTestResult();
		timerStart=false;
	}
	public void showStatistics() {
		long endTime=System.currentTimeMillis();
		float accuracy=0.0f;
		if (wordCount>0)
			accuracy=((float)correctCount)/wordCount;
		System.out.println(wordCount+"\t"+(endTime-startTime));
		float speed=((float)wordCount)*1000*60/(endTime-startTime);
		float adjSpeed=speed*accuracy;
		System.out.println("Speed: "+(int)speed+"\tAccuracy: "+((int)(accuracy*100))+
			"%\tNet Speed: "+(int)adjSpeed+" words per minute");
	}
	public void setStatistics() {
		long endTime=System.currentTimeMillis();
		float accuracy=0.0f;
		if (wordCount>0)
			accuracy=((float)correctCount)/wordCount;
		float speed=((float)wordCount)*1000*60/(endTime-startTime);
		float adjSpeed=speed*accuracy;
		StatPanel.set(""+wordCount, f2(speed), ""+((int)(accuracy*100))+" %", f2(adjSpeed));
	}
	public void showTestResult() {
		long endTime=System.currentTimeMillis();
		float accuracy=0.0f;
		if (wordCount>0)
			accuracy=((float)correctCount)/wordCount;
		float speed=((float)wordCount)*1000*60/(endTime-startTime);
		float adjSpeed=speed*accuracy;
		TestResult.addResult(title, ""+formatTime((endTime-startTime)/1000), f2(speed), 
			((int)(accuracy*100)), f2(adjSpeed));
		StoreResult.saveResult(title, (endTime-startTime)/1000, speed, accuracy*100, adjSpeed);
		resultDialog.show();
	}
	public void showWrongWords() {
		System.out.println("Wrongly typed words: "+wrongWord.size());
		for (int i=0; i<wrongWord.size(); i++)
			System.out.println(wrongWord.elementAt(i));
	}

	public static String f2(float val) {
		val=(int)(val*100);
		return ""+(val/100.0);
	}

	public static boolean isEndKey(int code) {
		if (KeyEvent.VK_END == code)
			return true;
		else	return false;
	}
	public static boolean ignoreKey(int code) {
		if (KeyEvent.VK_PAGE_UP <= code  && code <= KeyEvent.VK_DOWN)
			return true;
		if (code==KeyEvent.VK_UNDO || code==KeyEvent.VK_COPY || code==KeyEvent.VK_PASTE ||
				code==KeyEvent.VK_CUT)
			return true;
		
		return false;
	}
	public String formatTime(long time) {
		int min=(int)(time/60);
		int sec=(int)(time%60);
		String timeStr=min+":"+sec;
		if (sec<10)
			timeStr=min+":0"+sec;
		return timeStr;
	}

	public void setIsOwnFile(boolean isOwn) {isOwnFile=isOwn;}
	public void setTitle(String t) {title=t;}
	public void setResultDialog(Dialog res) {resultDialog=res;}
}

/*	Some control key definitions
VK_BACK_SPACE	8
VK_SPACE	32

VK_PAGE_UP	33
VK_PAGE_DOWN	34
VK_END	35
VK_HOME	36
VK_LEFT	37
VK_UP	38
VK_RIGHT	39
VK_DOWN	40

VK_COMMA	44
VK_MINUS	45
VK_PERIOD	46
VK_SLASH	47
VK_DELETE	127
VK_STOP	65480
VK_AGAIN	65481
VK_PROPS	65482
VK_UNDO	65483
VK_COPY	65485
VK_PASTE	65487
VK_FIND	65488
VK_CUT	65489

*/
