package ngc.typetest;

import java.awt.*;
import java.util.*;

/** this class is the main application class
 */

public class TypingTest extends Frame {
	public static void main(String []args) {
		String user="";
		if (args.length>0 && args[0].length()>0) {
			user=args[0]; 
			StoreResult.setUser(user);
		}
		System.out.println("User: "+user + args.length);
		TypingTest tt=new TypingTest();
	}

	protected TestDocument testDoc=null;
	protected TestArea testArea=null;
	protected DocumentManager docMgr=new DocumentManager();
	protected Dialog resultDia=null;
	protected Dialog aboutDialog=null, howtoDialog=null;
	protected String launchType="APPLICATION";
	public TypingTest() {
		setTitle("Typing Test");
		setSize(800,600);
		createMenu();
		initComponent();
		addWindowListener(new ExitListener());
		setResizable(false);
		setVisible(true);
		testDoc.changeFont();
		testArea.changeFont();
		createResultDialog();
		checkLogin();
	}
	public void setLaunchType(String type) {launchType=type;}
	public void initComponent() {
		testDoc=new TestDocument();
		add(testDoc,BorderLayout.NORTH);

		StatPanel statPanel=StatPanel.getStatPanel();
		add(statPanel, BorderLayout.CENTER);

		testArea=new TestArea(testDoc);
		add(testArea, BorderLayout.SOUTH);
		statPanel.setTestAction(testArea);
		statPanel.setTestArea(testArea);

		Thread th=new Thread(statPanel);
		th.start();
		readDocument();
	}
	public void createMenu() {
		MenuBar menuBar=new MenuBar();
		setMenuBar(menuBar);

		ApplicationAction aa=new ApplicationAction(this);
		Menu fileMenu=new Menu("File");
		menuBar.add(fileMenu);
		createFileMenu(fileMenu, aa);

		Menu testMenu=new Menu("Test");
		menuBar.add(testMenu);
		createTestMenu(testMenu, aa);

		DocumentAction da=new DocumentAction(this);
		Menu docMenu=new Menu("Document");
		menuBar.add(docMenu);
		createDocMenu(docMenu, da);

		Menu helpMenu=new Menu("Help");
		menuBar.add(helpMenu);
		createHelpMenu(helpMenu, aa);
	}
	public void createFileMenu(Menu fileMenu, ApplicationAction aa) {
		MenuItem  openMenu=new MenuItem("Open");
		fileMenu.add(openMenu);
		openMenu.addActionListener(aa);

		fileMenu.addSeparator();

		MenuItem exitMenu=new MenuItem("Exit");
		fileMenu.add(exitMenu);
		exitMenu.addActionListener(aa);
	}
	public void createTestMenu(Menu testMenu, ApplicationAction aa) {
		MenuItem restartMenu=new MenuItem("Restart");
		testMenu.add(restartMenu);
		testMenu.addActionListener(aa);

		testMenu.addSeparator();

		MenuItem min5=new MenuItem("5 min");
		min5.addActionListener(aa);
		testMenu.add(min5);

		MenuItem min10=new MenuItem("10 min");
		min10.addActionListener(aa);
		testMenu.add(min10);

		MenuItem min15=new MenuItem("15 min");
		min15.addActionListener(aa);
		testMenu.add(min15);
	}
	public void createDocMenu(Menu docMenu, DocumentAction da) {
		Vector docVect=docMgr.getDescription();
		for (int i=0; i<docVect.size(); i++) {
			String desc=(String)docVect.elementAt(i);
			MenuItem item=new MenuItem(desc);
			item.addActionListener(da);
			docMenu.add(item);
		}
	}

        public void fileOpen() {
                FileDialog openDialog=new FileDialog(this, "Open File", FileDialog.LOAD);
                String fs=System.getProperty("file.separator");
                String relDir=".";
                System.out.println("user.dir: "+System.getProperty("user.dir"));
                openDialog.setDirectory(System.getProperty("user.dir")+relDir);
                System.out.println("openDialg.getDirectory: "+openDialog.getDirectory());
                openDialog.setFile("*.txt");
                openDialog.show();

                String fileName = openDialog.getFile();
                if (fileName == null) {
                        System.out.println("No file has been selected");
                }
                else {
                        System.out.println("File: "+fileName);
                        String dir = openDialog.getDirectory();
                        System.out.println(dir);
			readFile(dir+fileName);
			testArea.setTitle("Own file: "+fileName);
			testArea.setIsOwnFile(true);
			hideResult();
                }
        }
	public void readFile(String filename) {
		testDoc.readFile99(filename);
		testArea.reset();
	}
	public void restart() {
		StatPanel.reset();
		testArea.reset();
		hideResult();
	}
	public void setTime(int tm) {
		StatPanel.setMaxTime(tm);
		StatPanel.reset();
		testArea.reset();
		hideResult();
	}
	public void readDocument() {
		Vector docVect=docMgr.getDescription();
		String desc=(String)docVect.elementAt(0);
		readDocument(desc);
	}
	public void readDocument(String desc) {
		String filename=docMgr.getFilename(desc);
		if (filename!=null) {
			testDoc.readFile(filename);
			testArea.reset();
			testArea.setTitle(desc);
			testArea.setIsOwnFile(false);
			hideResult();
		}
		else	System.out.println("document not found: "+desc);
	}
	public void createResultDialog() {
		resultDia=new Dialog(this);
		Panel resultTable=TestResult.getResultPanel();
		resultDia.add(resultTable);
		resultDia.setSize(550, 250);
		resultDia.setModal(false);
		resultDia.setTitle("Typing Test Result: "+StoreResult.getUser());

		testArea.setResultDialog(resultDia);
	}
	public void showResult() {if (resultDia!=null) resultDia.show();}
	public void hideResult() {if (resultDia!=null) resultDia.hide();}
	public void createHelpMenu(Menu helpMenu, ApplicationAction aa) {
		MenuItem about=new MenuItem("About");
		helpMenu.add(about);
		helpMenu.addActionListener(aa);
		
		MenuItem howto=new MenuItem("How to");
		helpMenu.add(howto);
		howto.addActionListener(aa);
	}
	public void showAboutDialog() {
		if (aboutDialog==null)
			createAboutDialog();
	}
	public void showHowtoDialog() {
		if (howtoDialog==null)
			createHowtoDialog();
	}
	public void createAboutDialog9() {
		aboutDialog=new Dialog(this, "About TypingTest");
		aboutDialog.setSize(400,200);
		TextArea ta=new TextArea(40,10);
		aboutDialog.add(ta);
		String text=	"TypingTest is written by Tony Lee\n"+
				"	tonylee@ri.sch.edu.sg\n"+
				"\n"+
				"This software uses TableBean by Thomas Studer\n"+
				"  http://www.conjective.ch/tablebean/e/index.html\n"+
				"\n"+
				"Test texts are obtained from the Gutenberg Project\n"+
				"  http://www.gutenberg.net";
		ta.setText(text);
		
	}

	public void createAboutDialog8() {
		aboutDialog=new Dialog(this, "About TypingTest");
		aboutDialog.setSize(400,200);
		String text=	"TypingTest is written by Tony Lee\n"+
				"	tonylee@ri.sch.edu.sg\n"+
				"\n"+
				"This software uses TableBean by Thomas Studer\n"+
				"  http://www.conjective.ch/tablebean/e/index.html\n"+
				"\n"+
				"Test texts are obtained from the Gutenberg Project\n"+
				"  http://www.gutenberg.net";
		javax.swing.JOptionPane opPane=new javax.swing.JOptionPane(text, 
			javax.swing.JOptionPane.INFORMATION_MESSAGE,
			javax.swing.JOptionPane.OK_CANCEL_OPTION);
		aboutDialog.add(opPane);
		
	}

	public void createAboutDialog() {
		String text=	"TypingTest is written by Tony Lee\n"+
				"	cybertny@gmail.com\n"+
				"\n"+
				"Test texts are obtained from the Gutenberg Project\n"+
				"  http://www.gutenberg.net"+
				"/n"+
				"      Version 0.5";
		javax.swing.JOptionPane.showMessageDialog(this, text, "About Typing Test", javax.swing.JOptionPane.INFORMATION_MESSAGE);
	}
	public void createHowtoDialog() {
		String text=	
			    "Type the highlighted word.  If you do not\n"+
				"follow the highlighted word, and you go\n"+
				"out of synchronisation, all the words you\n"+
				"typed will be considered incorrect!\n";
		javax.swing.JOptionPane.showMessageDialog(this, text, "How to", 
		javax.swing.JOptionPane.INFORMATION_MESSAGE);
	}
	public void checkLogin() {
		String user=StoreResult.getUser();
		if (user.trim().toUpperCase().equals("UNKNOWN")) {
			String text=	"Please login at the Netware prompt for results to be\n"+
				"sent to server.";
			javax.swing.JOptionPane.showMessageDialog(this, text, "Login Message", javax.swing.JOptionPane.INFORMATION_MESSAGE);
		}
		else setTitle("Typing Test: "+user);
	}

}

